/*=================================================================
 *
 *  lp.c
 *  Author: Orland Gonzalez
 *  Will act as an interface to the GLPK LP package.
 *
 *
 *=================================================================*/
/* $Revision: 1.10.6.2 $ */
#include <math.h>
#include "mex.h"
#include ".\include\glpk.h"

/* Constants */
#define MAX_NON_ZERO_ELEMENTS 60000

/* Input Arguments */

#define f_in       prhs[0]
#define A_in       prhs[1]
#define b_in       prhs[2]
#define lb_in      prhs[3]
#define ub_in      prhs[4]


/* Output Arguments */

#define	fval_out	plhs[0]
#define	x_out       plhs[1]
#define	sol_status_out  plhs[2]

/* Constants */

#define SOLUTION_OPTIMAL 1
#define SOLUTION_FEASIBLE 2
#define SOLUTION_INFEASIBLE 3
#define LP_HAS_NO_FEASIBLE_SOLUTION 4
#define LP_HAS_UNBOUNDED_SOLUTION 5
#define LP_FAILURE 6

void mexFunction( int nlhs, mxArray *plhs[], 
		  int nrhs, const mxArray*prhs[] ) { 
    double *f, *A, *b, *lb, *ub;
    int m, n; 
    int i, j, ctr, status_temp;
    
    /* Glpk variables */
    LPX *lp;
    int ia[1 + MAX_NON_ZERO_ELEMENTS], ja[1 + MAX_NON_ZERO_ELEMENTS];
    double ar[1 + MAX_NON_ZERO_ELEMENTS], *fval, *x, *sol_status;
    
    /* Check for proper number of arguments */    
    if (nrhs != 5) { 
        mexErrMsgTxt("Five input arguments required."); 
    } else if (nlhs > 3) {
        mexErrMsgTxt("Too many output arguments."); 
    }
    
    m = mxGetM(A_in);
    n = mxGetN(A_in);    
    
    /* Check input data */ 
    if ((m == 0) || (n == 0)) {
        mexErrMsgTxt("Empty problem."); 
    }
    
    if (m != mxGetM(b_in)) {
        mexErrMsgTxt("Rows of A should match rows of b."); 
    }
    
    if (n != mxGetN(f_in)) {
        mexErrMsgTxt("Columns of A should match columns of f."); 
    }
    
    if ((n != mxGetM(lb_in)) || (n != mxGetM(ub_in))) {
        mexErrMsgTxt("Columns of A should match rows of lb and ub."); 
    }
    
    if (!mxIsDouble(A_in) || !mxIsDouble(b_in) || !mxIsDouble(lb_in) 
            || !mxIsDouble(ub_in)) { 
        mexErrMsgTxt("All input parameters should be of type double."); 
    } 
    
    // Move matlab variables into interface variables
    f = mxGetPr(f_in); 
    A = mxGetPr(A_in);
    b = mxGetPr(b_in);
    lb = mxGetPr(lb_in);
    ub = mxGetPr(ub_in);
    
    /* Create matrices for the return arguments */ 
    fval_out = mxCreateDoubleMatrix(1, 1, mxREAL); 
    x_out = mxCreateDoubleMatrix(n, 1, mxREAL);         
    sol_status_out = mxCreateDoubleMatrix(1, 1, mxREAL);
    
    // Setup problem definition
    lp = lpx_create_prob();
    lpx_set_prob_name(lp, "glpk_matlab_interface");
    lpx_set_obj_dir(lp, LPX_MAX);
    
    // Set up the data for glpk
    lpx_add_rows(lp, m);
    lpx_add_cols(lp, n);
        
    for (i = 0; i < m; i++) {
        lpx_set_row_bnds(lp, i+1, LPX_FX, b[i], b[i]);        
    }
    for (j = 0; j < n; j++) {
        if (lb[j] == ub[j]) {
            lpx_set_col_bnds(lp, j+1, LPX_FX, lb[j], lb[j]);
        } else {
            lpx_set_col_bnds(lp, j+1, LPX_DB, lb[j], ub[j]);
        }
        lpx_set_obj_coef(lp, j+1, f[j]);
    }
    
    ctr = 0;
    for (i = 0; i < m; i++) {
        for (j = 0; j < n; j++) {
            if (A[(j*m) + i] != 0) {
                ctr++;
                
                if (ctr > MAX_NON_ZERO_ELEMENTS) {
                    mexErrMsgTxt("Problem scope is to large.");
                }
                ia[ctr] = i + 1;
                ja[ctr] = j + 1;
                ar[ctr] = A[(j*m) + i];
            }
        }
    }
    
    // Load data into LP structure of GLPK
    lpx_load_matrix(lp, ctr, ia, ja, ar);
    lpx_simplex(lp);
    
    // Retrieve output from Glpk and return it to matlab
    fval = mxGetPr(fval_out);
    x = mxGetPr(x_out);
    sol_status = mxGetPr(sol_status_out);
    
    fval[0] = lpx_get_obj_val(lp);
    for (j = 0; j < n; j++) {
        x[j] = lpx_get_col_prim(lp, j + 1);
    }
    
    status_temp = lpx_get_status(lp);
    if (status_temp == LPX_OPT) {
        sol_status[0] = SOLUTION_OPTIMAL;
    } else if (status_temp == LPX_FEAS) {
        sol_status[0] = SOLUTION_FEASIBLE;
    } else if (status_temp == LPX_INFEAS) {
        sol_status[0] = SOLUTION_INFEASIBLE;
    } else if (status_temp == LPX_NOFEAS) {
        sol_status[0] = LP_HAS_NO_FEASIBLE_SOLUTION;
    } else if (status_temp == LPX_UNBND) {
        sol_status[0] = LP_HAS_UNBOUNDED_SOLUTION;
    } else {
        sol_status[0] = LP_FAILURE;
    }
        
    
    lpx_delete_prob(lp);
    
    // Return output values to Matlab
    
    
    return;
    
}


